from .entity import EntityAPI
from .routing import Router
from .. import urls


class IncidentAPI(EntityAPI):
    """https://private.intel.amp.cisco.com/index.html#/Incident"""

    __router, route = Router.new()

    def __init__(self, request):
        super(IncidentAPI, self).__init__(request, '/ctia/incident')

    @route('status')
    def _perform(self, id_, payload, **kwargs):
        return self._post(
            urls.join(self._url, id_, 'status'),
            json=payload,
            **kwargs
        )

    @route('link')
    def _perform(self, id_, payload, **kwargs):
        return self._post(
            urls.join(self._url, id_, 'link'),
            json=payload,
            **kwargs
        )

    @route('sightings.incidents')
    def _perform(self, observable_type, observable_value, **kwargs):
        return self._get(
            urls.join(
                '/ctia',
                observable_type,
                observable_value,
                'sightings/incidents'
            ),
            **kwargs
        )



from six.moves.urllib.parse import quote

from .exceptions import RegionError


_url_patterns_by_api_family = {
    'visibility': 'https://visibility{region}.amp.cisco.com',
    'private_intel': 'https://private.intel{region}.amp.cisco.com',
    'global_intel': 'https://intel{region}.amp.cisco.com',
}


def _url_for_region(url_pattern, region):
    return url_pattern.format(region='.' + region if region != '' else '')


_urls_by_region = {
    region: {
        api_family: _url_for_region(url_pattern, region)
        for api_family, url_pattern in _url_patterns_by_api_family.items()
    }
    for region in (
        '',
        'eu',
        'apjc',
    )
}


def url_for(region, family):
    if region is None:
        region = ''

    if region not in _urls_by_region:
        # Use `repr` to make each region enclosed in quotes.
        raise RegionError(
            'Invalid region {}, must be one of: {}.'.format(
                repr(region),
                ', '.join(map(repr, _urls_by_region.keys())),
            )
        )

    return _urls_by_region[region][family]


def join(base, *parts):
    return base.rstrip('/') + '/' + '/'.join(
        quote(str(part).strip('/'), safe='')
        for part in parts
    )



# Make the classes below importable from the `.request` subpackage directly.
from .authorized import AuthorizedRequest
from .logged import LoggedRequest
from .proxied import ProxiedRequest
from .relative import RelativeRequest
from .response import Response
from .standard import StandardRequest



from .api.enrich import EnrichAPI
from .api.inspect import InspectAPI
from .api.intel import IntelAPI
from .api.response import ResponseAPI
from .request.authorized import AuthorizedRequest
from .request.logged import LoggedRequest
from .request.proxied import ProxiedRequest
from .request.relative import RelativeRequest
from .request.standard import StandardRequest
from .urls import url_for


class ThreatResponse(object):

    def __init__(self, client_id, client_password, **options):
        credentials = (client_id, client_password)

        proxy = options.get('proxy')
        logger = options.get('logger')
        region = options.get('region')

        request = ProxiedRequest(proxy) if proxy else StandardRequest()
        request = LoggedRequest(request, logger) if logger else request
        request = AuthorizedRequest(request, *credentials, region=region)

        def request_for(family):
            return RelativeRequest(request, url_for(region, family))

        self._inspect = InspectAPI(request_for('visibility'))
        self._enrich = EnrichAPI(request_for('visibility'))
        self._response = ResponseAPI(request_for('visibility'))
        self._private_intel = IntelAPI(request_for('private_intel'))
        self._global_intel = IntelAPI(request_for('global_intel'))

    @property
    def inspect(self):
        return self._inspect

    @property
    def enrich(self):
        return self._enrich

    @property
    def response(self):
        return self._response

    @property
    def private_intel(self):
        return self._private_intel

    @property
    def global_intel(self):
        return self._global_intel



from .entity import EntityAPI
from .routing import Router
from .. import urls


class EventAPI(EntityAPI):
    """https://private.intel.amp.cisco.com/index.html#/Event"""

    __router, route = Router.new()

    def __init__(self, request):
        super(EventAPI, self).__init__(request, '/ctia/event')

    @route('history')
    def _perform(self, id_, **kwargs):
        return self._get(
            urls.join(self._url, 'history', id_),
            **kwargs
        )
